@@ -14,16 +14,22 @@ CXX_GUARD_START
 #include <mgba/internal/gb/gb.h>
 #include <mgba/internal/gb/video.h>
 
+struct GBVideoRendererSprite {
+	struct GBObj obj;
+	int8_t index;
+};
+
 struct GBVideoSoftwareRenderer {
 	struct GBVideoRenderer d;
 
 	color_t* outputBuffer;
 	int outputBufferStride;
 
-	uint8_t row[GB_VIDEO_HORIZONTAL_PIXELS + 8];
+	// TODO: Implement the pixel FIFO
+	uint16_t row[GB_VIDEO_HORIZONTAL_PIXELS + 8];
 
-	color_t palette[128];
-	uint8_t lookup[64];
+	color_t palette[192];
+	uint8_t lookup[192];
 
 	uint32_t* temporaryBuffer;
 
@@ -40,6 +46,9 @@ struct GBVideoSoftwareRenderer {
 	GBRegisterLCDC lcdc;
 	enum GBModel model;
 
+	struct GBVideoRendererSprite obj[GB_VIDEO_MAX_LINE_OBJ];
+	int objMax;
+
 	int16_t objOffsetX;
 	int16_t objOffsetY;
 	int16_t offsetScx;
@@ -51,6 +60,9 @@ struct GBVideoSoftwareRenderer {
 	uint8_t sgbPacket[128];
 	uint8_t sgbCommandHeader;
 	bool sgbBorders;
+	uint32_t sgbBorderMask[18];
+
+	uint8_t lastHighlightAmount;
 };
 
 void GBVideoSoftwareRendererCreate(struct GBVideoSoftwareRenderer*);
