@@ -46,8 +46,8 @@ static void _redoCacheSize(struct mTileCache* cache) {
 	unsigned tiles = mTileCacheSystemInfoGetMaxTiles(cache->sysConfig);
 	cache->cache = anonymousMemoryMap(8 * 8 * sizeof(color_t) * tiles * size);
 	cache->status = anonymousMemoryMap(tiles * size * sizeof(*cache->status));
-	cache->globalPaletteVersion = malloc(size * sizeof(*cache->globalPaletteVersion));
-	cache->palette = malloc(size * bpp * sizeof(*cache->palette));
+	cache->globalPaletteVersion = calloc(size, sizeof(*cache->globalPaletteVersion));
+	cache->palette = calloc(size * bpp, sizeof(*cache->palette));
 }
 
 void mTileCacheConfigure(struct mTileCache* cache, mTileCacheConfiguration config) {
