@@ -8,6 +8,7 @@
 void mTimingInit(struct mTiming* timing, int32_t* relativeCycles, int32_t* nextEvent) {
 	timing->root = NULL;
 	timing->reroot = NULL;
+	timing->globalCycles = 0;
 	timing->masterCycles = 0;
 	timing->relativeCycles = relativeCycles;
 	timing->nextEvent = nextEvent;
@@ -20,9 +21,18 @@ void mTimingDeinit(struct mTiming* timing) {
 void mTimingClear(struct mTiming* timing) {
 	timing->root = NULL;
 	timing->reroot = NULL;
+	timing->globalCycles = 0;
 	timing->masterCycles = 0;
 }
 
+void mTimingInterrupt(struct mTiming* timing) {
+	if (!timing->root) {
+		return;
+	}
+	timing->reroot = timing->root;
+	timing->root = NULL;
+}
+
 void mTimingSchedule(struct mTiming* timing, struct mTimingEvent* event, int32_t when) {
 	int32_t nextEvent = when + *timing->relativeCycles;
 	event->when = nextEvent + timing->masterCycles;
@@ -48,6 +58,10 @@ void mTimingSchedule(struct mTiming* timing, struct mTimingEvent* event, int32_t
 	*previous = event;
 }
 
+void mTimingScheduleAbsolute(struct mTiming* timing, struct mTimingEvent* event, int32_t when) {
+	mTimingSchedule(timing, event, when - mTimingCurrentTime(timing));
+}
+
 void mTimingDeschedule(struct mTiming* timing, struct mTimingEvent* event) {
 	if (timing->reroot) {
 		timing->root = timing->reroot;
@@ -94,7 +108,10 @@ int32_t mTimingTick(struct mTiming* timing, int32_t cycles) {
 	if (timing->reroot) {
 		timing->root = timing->reroot;
 		timing->reroot = NULL;
-		*timing->nextEvent = mTimingNextEvent(timing); 
+		*timing->nextEvent = mTimingNextEvent(timing);
+		if (*timing->nextEvent <= 0) {
+			return mTimingTick(timing, 0);
+		}
 	}
 	return *timing->nextEvent;
 }
@@ -103,6 +120,10 @@ int32_t mTimingCurrentTime(const struct mTiming* timing) {
 	return timing->masterCycles + *timing->relativeCycles;
 }
 
+uint64_t mTimingGlobalTime(const struct mTiming* timing) {
+	return timing->globalCycles + *timing->relativeCycles;
+}
+
 int32_t mTimingNextEvent(struct mTiming* timing) {
 	struct mTimingEvent* next = timing->root;
 	if (!next) {
