@@ -8,15 +8,17 @@
 #include <mgba/internal/debugger/symbols.h>
 
 #include <mgba/core/core.h>
-#include <mgba/core/timing.h>
-#include <mgba/core/version.h>
-#include <mgba/internal/debugger/parser.h>
-#include <mgba-util/string.h>
-#include <mgba-util/vfs.h>
-
 #ifdef ENABLE_SCRIPTING
 #include <mgba/core/scripting.h>
 #endif
+#include <mgba/core/timing.h>
+#include <mgba/core/version.h>
+#include <mgba/internal/debugger/parser.h>
+#ifdef USE_ELF
+#include <mgba-util/elf-read.h>
+#endif
+#include <mgba-util/string.h>
+#include <mgba-util/vfs.h>
 
 #if !defined(NDEBUG) && !defined(_WIN32)
 #include <signal.h>
@@ -74,6 +76,7 @@ static void _source(struct CLIDebugger*, struct CLIDebugVector*);
 static void _backtrace(struct CLIDebugger*, struct CLIDebugVector*);
 static void _finish(struct CLIDebugger*, struct CLIDebugVector*);
 static void _setStackTraceMode(struct CLIDebugger*, struct CLIDebugVector*);
+static void _loadSymbols(struct CLIDebugger*, struct CLIDebugVector*);
 static void _setSymbol(struct CLIDebugger*, struct CLIDebugVector*);
 static void _findSymbol(struct CLIDebugger*, struct CLIDebugVector*);
 
@@ -101,6 +104,7 @@ static struct CLIDebuggerCommandSummary _debuggerCommands[] = {
 	{ "stack", _setStackTraceMode, "S", "Change the stack tracing mode" },
 	{ "status", _printStatus, "", "Print the current status" },
 	{ "symbol", _findSymbol, "I", "Find the symbol name for an address" },
+	{ "load-symbols", _loadSymbols, "S", "Load symbols from an external file" },
 	{ "trace", _trace, "Is", "Trace a number of instructions" },
 	{ "w/1", _writeByte, "II", "Write a byte at a specified offset" },
 	{ "w/2", _writeHalfword, "II", "Write a halfword at a specified offset" },
@@ -133,6 +137,7 @@ static struct CLIDebuggerCommandAlias _debuggerCommandAliases[] = {
 	{ "h", "help" },
 	{ "i", "status" },
 	{ "info", "status" },
+	{ "loadsyms", "load-symbols" },
 	{ "lb", "listb" },
 	{ "lw", "listw" },
 	{ "n", "next" },
@@ -221,11 +226,9 @@ static bool _parseExpression(struct mDebugger* debugger, struct CLIDebugVector*
 	}
 	if (!mDebuggerEvaluateParseTree(debugger, tree, intValue, segmentValue)) {
 		parseFree(tree);
-		free(tree);
 		return false;
 	}
 	parseFree(tree);
-	free(tree);
 	return true;
 }
 
@@ -323,16 +326,24 @@ static void _printHelp(struct CLIDebugger* debugger, struct CLIDebugVector* dv)
 		debugger->backend->printf(debugger->backend, "Generic commands:\n");
 		_printCommands(debugger, _debuggerCommands, _debuggerCommandAliases);
 		if (debugger->system) {
-			debugger->backend->printf(debugger->backend, "\n%s commands:\n", debugger->system->platformName);
-			_printCommands(debugger, debugger->system->platformCommands, debugger->system->platformCommandAliases);
-			debugger->backend->printf(debugger->backend, "\n%s commands:\n", debugger->system->name);
-			_printCommands(debugger, debugger->system->commands, debugger->system->commandAliases);
+			if (debugger->system->platformCommands) {
+				debugger->backend->printf(debugger->backend, "\n%s commands:\n", debugger->system->platformName);
+				_printCommands(debugger, debugger->system->platformCommands, debugger->system->platformCommandAliases);
+			}
+			if (debugger->system->commands) {
+				debugger->backend->printf(debugger->backend, "\n%s commands:\n", debugger->system->name);
+				_printCommands(debugger, debugger->system->commands, debugger->system->commandAliases);
+			}
 		}
 	} else {
 		_printCommandSummary(debugger, dv->charValue, _debuggerCommands, _debuggerCommandAliases);
 		if (debugger->system) {
-			_printCommandSummary(debugger, dv->charValue, debugger->system->platformCommands, debugger->system->platformCommandAliases);
-			_printCommandSummary(debugger, dv->charValue, debugger->system->commands, debugger->system->commandAliases);
+			if (debugger->system->platformCommands) {
+				_printCommandSummary(debugger, dv->charValue, debugger->system->platformCommands, debugger->system->platformCommandAliases);
+			}
+			if (debugger->system->commands) {
+				_printCommandSummary(debugger, dv->charValue, debugger->system->commands, debugger->system->commandAliases);
+			}
 		}
 	}
 }
@@ -447,7 +458,7 @@ static void _writeRegister(struct CLIDebugger* debugger, struct CLIDebugVector*
 		debugger->backend->printf(debugger->backend, "%s\n", ERROR_INVALID_ARGS);
 		return;
 	}
-	if (!debugger->d.platform->setRegister(debugger->d.platform, dv->charValue, dv->next->intValue)) {
+	if (!debugger->d.core->writeRegister(debugger->d.core, dv->charValue, &dv->next->intValue)) {
 		debugger->backend->printf(debugger->backend, "%s\n", ERROR_INVALID_ARGS);
 	}
 }
@@ -585,7 +596,7 @@ static struct ParseTree* _parseTree(const char** string) {
 	}
 	struct ParseTree* tree = NULL;
 	if (!error) {
-		tree = malloc(sizeof(*tree));
+		tree = parseTreeCreate();
 		parseLexedExpression(tree, &lv);
 	}
 	lexFree(&lv);
@@ -594,7 +605,6 @@ static struct ParseTree* _parseTree(const char** string) {
 	if (error) {
 		if (tree) {
 			parseFree(tree);
-			free(tree);
 		}
 		return NULL;
 	} else {
@@ -794,17 +804,16 @@ struct CLIDebugVector* CLIDVParse(struct CLIDebugger* debugger, const char* stri
 		dvTemp.type = CLIDV_ERROR_TYPE;
 	}
 
-	struct ParseTree tree;
-	parseLexedExpression(&tree, &lv);
-	if (tree.token.type == TOKEN_ERROR_TYPE) {
+	struct ParseTree* tree = parseTreeCreate();
+	if (!parseLexedExpression(tree, &lv)) {
 		dvTemp.type = CLIDV_ERROR_TYPE;
 	} else {
-		if (!mDebuggerEvaluateParseTree(&debugger->d, &tree, &dvTemp.intValue, &dvTemp.segmentValue)) {
+		if (!mDebuggerEvaluateParseTree(&debugger->d, tree, &dvTemp.intValue, &dvTemp.segmentValue)) {
 			dvTemp.type = CLIDV_ERROR_TYPE;
 		}
 	}
 
-	parseFree(&tree);
+	parseFree(tree);
 
 	lexFree(&lv);
 	LexVectorDeinit(&lv);
@@ -980,8 +989,10 @@ bool CLIDebuggerRunCommand(struct CLIDebugger* debugger, const char* line, size_
 	}
 	int result = _tryCommands(debugger, _debuggerCommands, _debuggerCommandAliases, line, cmdLength, args, count - cmdLength - 1);
 	if (result < 0 && debugger->system) {
-		result = _tryCommands(debugger, debugger->system->commands, debugger->system->commandAliases, line, cmdLength, args, count - cmdLength - 1);
-		if (result < 0) {
+		if (debugger->system->commands) {
+			result = _tryCommands(debugger, debugger->system->commands, debugger->system->commandAliases, line, cmdLength, args, count - cmdLength - 1);
+		}
+		if (result < 0 && debugger->system->platformCommands) {
 			result = _tryCommands(debugger, debugger->system->platformCommands, debugger->system->platformCommandAliases, line, cmdLength, args, count - cmdLength - 1);
 		}
 	}
@@ -994,21 +1005,33 @@ bool CLIDebuggerRunCommand(struct CLIDebugger* debugger, const char* line, size_
 static void _commandLine(struct mDebugger* debugger) {
 	struct CLIDebugger* cliDebugger = (struct CLIDebugger*) debugger;
 	const char* line;
-		size_t len;
-	_printStatus(cliDebugger, 0);
+	size_t len;
+	if (cliDebugger->skipStatus) {
+		cliDebugger->skipStatus = false;
+	} else {
+		_printStatus(cliDebugger, 0);
+	}
 	while (debugger->state == DEBUGGER_PAUSED) {
 		line = cliDebugger->backend->readline(cliDebugger->backend, &len);
 		if (!line || len == 0) {
 			debugger->state = DEBUGGER_SHUTDOWN;
 			return;
 		}
+		if (line[0] == '\033') {
+			cliDebugger->skipStatus = true;
+			return;
+		}
 		if (line[0] == '\n') {
 			line = cliDebugger->backend->historyLast(cliDebugger->backend, &len);
 			if (line && len) {
 				CLIDebuggerRunCommand(cliDebugger, line, len);
 			}
 		} else {
-			CLIDebuggerRunCommand(cliDebugger, line, len);
+			if (line[0] == '#') {
+				cliDebugger->skipStatus = true;
+			} else {
+				CLIDebuggerRunCommand(cliDebugger, line, len);
+			}
 			cliDebugger->backend->historyAppend(cliDebugger->backend, line);
 		}
 	}
@@ -1019,6 +1042,7 @@ static void _reportEntry(struct mDebugger* debugger, enum mDebuggerEntryReason r
 	if (cliDebugger->traceRemaining > 0) {
 		cliDebugger->traceRemaining = 0;
 	}
+	cliDebugger->skipStatus = false;
 	switch (reason) {
 	case DEBUGGER_ENTER_MANUAL:
 	case DEBUGGER_ENTER_ATTACHED:
@@ -1077,6 +1101,7 @@ static void _cliDebuggerInit(struct mDebugger* debugger) {
 	struct CLIDebugger* cliDebugger = (struct CLIDebugger*) debugger;
 	cliDebugger->traceRemaining = 0;
 	cliDebugger->traceVf = NULL;
+	cliDebugger->skipStatus = false;
 	cliDebugger->backend->init(cliDebugger->backend);
 	if (cliDebugger->system && cliDebugger->system->init) {
 		cliDebugger->system->init(cliDebugger->system);
@@ -1119,12 +1144,21 @@ static void _cliDebuggerCustom(struct mDebugger* debugger) {
 	}
 }
 
+static void _cliDebuggerInterrupt(struct mDebugger* debugger) {
+	struct CLIDebugger* cliDebugger = (struct CLIDebugger*) debugger;
+	if (cliDebugger->backend->interrupt) {
+		cliDebugger->backend->interrupt(cliDebugger->backend);
+	}
+}
+
 void CLIDebuggerCreate(struct CLIDebugger* debugger) {
 	debugger->d.init = _cliDebuggerInit;
 	debugger->d.deinit = _cliDebuggerDeinit;
 	debugger->d.custom = _cliDebuggerCustom;
 	debugger->d.paused = _commandLine;
+	debugger->d.update = NULL;
 	debugger->d.entered = _reportEntry;
+	debugger->d.interrupt = _cliDebuggerInterrupt;
 	debugger->d.type = DEBUGGER_CLI;
 
 	debugger->system = NULL;
@@ -1268,6 +1302,40 @@ static void _setStackTraceMode(struct CLIDebugger* debugger, struct CLIDebugVect
 	}
 }
 
+static void _loadSymbols(struct CLIDebugger* debugger, struct CLIDebugVector* dv) {
+	struct mDebuggerSymbols* symbolTable = debugger->d.core->symbolTable;
+	if (!symbolTable) {
+		debugger->backend->printf(debugger->backend, "No symbol table available.\n");
+		return;
+	}
+	if (!dv || dv->next) {
+		debugger->backend->printf(debugger->backend, "%s\n", ERROR_MISSING_ARGS);
+		return;
+	}
+	if (dv->type != CLIDV_CHAR_TYPE) {
+		debugger->backend->printf(debugger->backend, "%s\n", ERROR_INVALID_ARGS);
+		return;
+	}
+	struct VFile* vf = VFileOpen(dv->charValue, O_RDONLY);
+	if (!vf) {
+		debugger->backend->printf(debugger->backend, "%s\n", "Could not open symbol file");
+		return;
+	}
+#ifdef USE_ELF
+	struct ELF* elf = ELFOpen(vf);
+	if (elf) {
+#ifdef USE_DEBUGGERS
+		mCoreLoadELFSymbols(symbolTable, elf);
+#endif
+		ELFClose(elf);
+	} else
+#endif
+	{
+		mDebuggerLoadARMIPSSymbols(symbolTable, vf);
+	}
+	vf->close(vf);
+}
+
 static void _setSymbol(struct CLIDebugger* debugger, struct CLIDebugVector* dv) {
 	struct mDebuggerSymbols* symbolTable = debugger->d.core->symbolTable;
 	if (!symbolTable) {
