@@ -11,6 +11,8 @@
 
 DEFINE_VECTOR(LexVector, struct Token);
 
+DEFINE_VECTOR(IntList, int32_t);
+
 enum LexState {
 	LEX_ERROR = -1,
 	LEX_ROOT = 0,
@@ -493,16 +495,21 @@ static const int _operatorPrecedence[] = {
 	[OP_DEREFERENCE] = 2,
 };
 
-static struct ParseTree* _parseTreeCreate() {
+struct ParseTree* parseTreeCreate(void) {
 	struct ParseTree* tree = malloc(sizeof(struct ParseTree));
 	tree->token.type = TOKEN_ERROR_TYPE;
-	tree->rhs = 0;
-	tree->lhs = 0;
+	tree->p = NULL;
+	tree->rhs = NULL;
+	tree->lhs = NULL;
+	tree->precedence = INT_MAX;
 	return tree;
 }
 
-static size_t _parseExpression(struct ParseTree* tree, struct LexVector* lv, size_t i, int precedence, int* openParens) {
-	struct ParseTree* newTree = 0;
+static size_t _parseExpression(struct ParseTree* tree, struct LexVector* lv, int* openParens) {
+	struct ParseTree* newTree = NULL;
+	bool pop = false;
+	int precedence = INT_MAX;
+	size_t i = 0;
 	while (i < LexVectorSize(lv)) {
 		struct Token* token = LexVectorGetPointer(lv, i);
 		int newPrecedence;
@@ -517,27 +524,36 @@ static size_t _parseExpression(struct ParseTree* tree, struct LexVector* lv, siz
 				++i;
 			} else {
 				tree->token.type = TOKEN_ERROR_TYPE;
-				return i + 1;
+				++i;
+				pop = true;
 			}
 			break;
 		case TOKEN_SEGMENT_TYPE:
-			tree->lhs = _parseTreeCreate();
+			tree->lhs = parseTreeCreate();
 			tree->lhs->token.type = TOKEN_UINT_TYPE;
 			tree->lhs->token.uintValue = token->uintValue;
-			tree->rhs = _parseTreeCreate();
+			tree->lhs->p = tree;
+			tree->lhs->precedence = precedence;
+			tree->rhs = parseTreeCreate();
+			tree->rhs->p = tree;
+			tree->rhs->precedence = precedence;
 			tree->token.type = TOKEN_SEGMENT_TYPE;
-			i = _parseExpression(tree->rhs, lv, i + 1, precedence, openParens);
+			tree = tree->rhs;
+			++i;
 			break;
 		case TOKEN_OPEN_PAREN_TYPE:
 			++*openParens;
-			i = _parseExpression(tree, lv, i + 1, INT_MAX, openParens);
+			precedence = INT_MAX;
+			++i;
 			break;
 		case TOKEN_CLOSE_PAREN_TYPE:
 			if (*openParens <= 0) {
 				tree->token.type = TOKEN_ERROR_TYPE;
 			}
 			--*openParens;
-			return i + 1;
+			++i;
+			pop = true;
+			break;
 		case TOKEN_OPERATOR_TYPE:
 			if (tree->token.type == TOKEN_ERROR_TYPE) {
 				switch (token->operatorValue) {
@@ -556,45 +572,71 @@ static size_t _parseExpression(struct ParseTree* tree, struct LexVector* lv, siz
 			}
 			newPrecedence = _operatorPrecedence[token->operatorValue];
 			if (newPrecedence < precedence) {
-				newTree = _parseTreeCreate();
-				*newTree = *tree;
-				tree->lhs = newTree;
-				tree->rhs = _parseTreeCreate();
-				tree->token = *token;
-				i = _parseExpression(tree->rhs, lv, i + 1, newPrecedence, openParens);
-				if (tree->token.type == TOKEN_ERROR_TYPE) {
-					tree->token.type = TOKEN_ERROR_TYPE;
+				newTree = parseTreeCreate();
+				memcpy(newTree, tree, sizeof(*tree));
+				if (newTree->lhs) {
+					newTree->lhs->p = newTree;
 				}
+				if (newTree->rhs) {
+					newTree->rhs->p = newTree;
+				}
+				newTree->p = tree;
+				tree->lhs = newTree;
+				tree->rhs = parseTreeCreate();
+				tree->rhs->p = tree;
+				tree->rhs->precedence = newPrecedence;
+				precedence = newPrecedence;
+				tree->token = *token;
+				tree = tree->rhs;
+				++i;
 			} else {
-				return i;
+				pop = true;
 			}
 			break;
 		case TOKEN_ERROR_TYPE:
 			tree->token.type = TOKEN_ERROR_TYPE;
-			return i + 1;
+			++i;
+			pop = true;
+			break;
+		}
+
+		if (pop) {
+			if (tree->token.type == TOKEN_ERROR_TYPE && tree->p) {
+				tree->p->token.type = TOKEN_ERROR_TYPE;
+			}
+			tree = tree->p;
+			pop = false;
+			if (!tree) {
+				break;
+			} else {
+				precedence = tree->precedence;
+			}
 		}
 	}
 
 	return i;
 }
 
-void parseLexedExpression(struct ParseTree* tree, struct LexVector* lv) {
+bool parseLexedExpression(struct ParseTree* tree, struct LexVector* lv) {
 	if (!tree) {
-		return;
+		return false;
 	}
 
 	tree->token.type = TOKEN_ERROR_TYPE;
-	tree->lhs = 0;
-	tree->rhs = 0;
+	tree->lhs = NULL;
+	tree->rhs = NULL;
+	tree->p = NULL;
+	tree->precedence = INT_MAX;
 
 	int openParens = 0;
-	_parseExpression(tree, lv, 0, INT_MAX, &openParens);
+	_parseExpression(tree, lv, &openParens);
 	if (openParens) {
 		if (tree->token.type == TOKEN_IDENTIFIER_TYPE) {
 			free(tree->token.identifierValue);
 		}
 		tree->token.type = TOKEN_ERROR_TYPE;
 	}
+	return tree->token.type != TOKEN_ERROR_TYPE;
 }
 
 void lexFree(struct LexVector* lv) {
@@ -607,23 +649,40 @@ void lexFree(struct LexVector* lv) {
 	}
 }
 
-void parseFree(struct ParseTree* tree) {
-	if (!tree) {
-		return;
-	}
-
-	if (tree->lhs) {
-		parseFree(tree->lhs);
-		free(tree->lhs);
-	}
-	if (tree->rhs) {
-		parseFree(tree->rhs);
-		free(tree->rhs);
-	}
-
+static void _freeTree(struct ParseTree* tree) {
 	if (tree->token.type == TOKEN_IDENTIFIER_TYPE) {
 		free(tree->token.identifierValue);
 	}
+	free(tree);
+}
+
+void parseFree(struct ParseTree* tree) {
+	while (tree) {
+		if (tree->lhs) {
+			tree = tree->lhs;
+			continue;
+		}
+		if (tree->rhs) {
+			tree = tree->rhs;
+			continue;
+		}
+		if (tree->p) {
+			if (tree->p->lhs == tree) {
+				tree = tree->p;
+				_freeTree(tree->lhs);
+				tree->lhs = NULL;
+			} else if (tree->p->rhs == tree) {
+				tree = tree->p;
+				_freeTree(tree->rhs);
+				tree->rhs = NULL;
+			} else {
+				abort();
+			}
+		} else {
+			_freeTree(tree);
+			break;
+		}
+	}
 }
 
 static bool _performOperation(struct mDebugger* debugger, enum Operation operation, int32_t current, int32_t next, int32_t* value, int* segment) {
@@ -721,56 +780,125 @@ bool mDebuggerEvaluateParseTree(struct mDebugger* debugger, struct ParseTree* tr
 	if (!value) {
 		return false;
 	}
-	int32_t lhs, rhs;
-	switch (tree->token.type) {
-	case TOKEN_UINT_TYPE:
-		if (segment) {
-			*segment = -1;
-		}
-		*value = tree->token.uintValue;
-		return true;
-	case TOKEN_SEGMENT_TYPE:
-		if (!mDebuggerEvaluateParseTree(debugger, tree->rhs, value, segment)) {
-			return false;
-		}
-		return mDebuggerEvaluateParseTree(debugger, tree->lhs, segment, NULL);
-	case TOKEN_OPERATOR_TYPE:
-		switch (tree->token.operatorValue) {
-		case OP_ASSIGN:
-		case OP_ADD:
-		case OP_SUBTRACT:
-		case OP_MULTIPLY:
-		case OP_DIVIDE:
-		case OP_MODULO:
-		case OP_AND:
-		case OP_OR:
-		case OP_XOR:
-		case OP_LESS:
-		case OP_GREATER:
-		case OP_EQUAL:
-		case OP_NOT_EQUAL:
-		case OP_LOGICAL_AND:
-		case OP_LOGICAL_OR:
-		case OP_LE:
-		case OP_GE:
-		case OP_SHIFT_L:
-		case OP_SHIFT_R:
-			if (!mDebuggerEvaluateParseTree(debugger, tree->lhs, &lhs, segment)) {
-				return false;
-			}
-			// Fall through
-		default:
-			if (!mDebuggerEvaluateParseTree(debugger, tree->rhs, &rhs, segment)) {
-				return false;
+	struct IntList stack;
+	int nextBranch;
+	bool ok = true;
+	int32_t tmpVal = 0;
+	int32_t tmpSegment = -1;
+
+	IntListInit(&stack, 0);
+	while (ok) {
+		switch (tree->token.type) {
+		case TOKEN_UINT_TYPE:
+			nextBranch = 2;
+			tmpSegment = -1;
+			tmpVal = tree->token.uintValue;
+			break;
+		case TOKEN_SEGMENT_TYPE:
+			nextBranch = 0;
+			break;
+		case TOKEN_OPERATOR_TYPE:
+			switch (tree->token.operatorValue) {
+			case OP_ASSIGN:
+			case OP_ADD:
+			case OP_SUBTRACT:
+			case OP_MULTIPLY:
+			case OP_DIVIDE:
+			case OP_MODULO:
+			case OP_AND:
+			case OP_OR:
+			case OP_XOR:
+			case OP_LESS:
+			case OP_GREATER:
+			case OP_EQUAL:
+			case OP_NOT_EQUAL:
+			case OP_LOGICAL_AND:
+			case OP_LOGICAL_OR:
+			case OP_LE:
+			case OP_GE:
+			case OP_SHIFT_L:
+			case OP_SHIFT_R:
+				nextBranch = 0;
+				break;
+			default:
+				nextBranch = 1;
+				break;
 			}
 			break;
+		case TOKEN_IDENTIFIER_TYPE:
+			if (!mDebuggerLookupIdentifier(debugger, tree->token.identifierValue, &tmpVal, &tmpSegment)) {
+				ok = false;
+			}
+			nextBranch = 2;
+			break;
+		case TOKEN_ERROR_TYPE:
+		default:
+			ok = false;
+			break;
+		}
+		if (!ok) {
+			break;
+		}
+
+		bool gotTree = false;
+		while (!gotTree && tree) {
+			int32_t lhs, rhs;
+
+			switch (nextBranch) {
+			case 0:
+				*IntListAppend(&stack) = tmpVal;
+				*IntListAppend(&stack) = tmpSegment;
+				*IntListAppend(&stack) = nextBranch;
+				tree = tree->lhs;
+				gotTree = true;
+				break;
+			case 1:
+				*IntListAppend(&stack) = tmpVal;
+				*IntListAppend(&stack) = tmpSegment;
+				*IntListAppend(&stack) = nextBranch;
+				tree = tree->rhs;
+				gotTree = true;
+				break;
+			case 2:
+				if (!IntListSize(&stack)) {
+					tree = NULL;
+					break;
+				}
+				nextBranch = *IntListGetPointer(&stack, IntListSize(&stack) - 1);
+				IntListResize(&stack, -1);
+				tree = tree->p;
+				if (nextBranch == 0) {
+					++nextBranch;
+				} else if (tree) {
+					nextBranch = 2;
+					switch (tree->token.type) {
+					case TOKEN_OPERATOR_TYPE:
+						rhs = tmpVal;
+						lhs = *IntListGetPointer(&stack, IntListSize(&stack) - 2);
+						tmpSegment = *IntListGetPointer(&stack, IntListSize(&stack) - 1);
+						ok = _performOperation(debugger, tree->token.operatorValue, lhs, rhs, &tmpVal, &tmpSegment);
+						break;
+					case TOKEN_SEGMENT_TYPE:
+						tmpSegment = *IntListGetPointer(&stack, IntListSize(&stack) - 2);
+						break;
+					default:
+						break;
+					}
+				}
+				IntListResize(&stack, -2);
+				break;
+			}
+		}
+		if (!tree) {
+			break;
 		}
-		return _performOperation(debugger, tree->token.operatorValue, lhs, rhs, value, segment);
-	case TOKEN_IDENTIFIER_TYPE:
-		return mDebuggerLookupIdentifier(debugger, tree->token.identifierValue, value, segment);
-	case TOKEN_ERROR_TYPE:
-	default:
-		break;
 	}
-	return false;
+	IntListDeinit(&stack);
+	if (ok) {
+		*value = tmpVal;
+		if (segment) {
+			*segment = tmpSegment;
+		}
+	}
+	return ok;
 }
