@@ -12,6 +12,7 @@
 #include <mgba/internal/gba/gba.h>
 #ifdef M_CORE_GB
 #include <mgba/internal/gb/gb.h>
+#include <mgba/internal/gb/overrides.h>
 #endif
 #include <mgba-util/gui/file-select.h>
 #include <mgba-util/gui/menu.h>
@@ -21,6 +22,11 @@
 #define GUI_MAX_INPUTS 7
 #endif
 
+enum {
+	CONFIG_REMAP,
+	CONFIG_SAVE,
+};
+
 static bool _biosNamed(const char* name) {
 	char ext[PATH_MAX + 1] = {};
 	separatePath(name, NULL, NULL, ext);
@@ -40,10 +46,11 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 		.index = 0,
 		.background = &runner->background.d
 	};
+	size_t i;
 	GUIMenuItemListInit(&menu.items, 0);
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Frameskip",
-		.data = "frameskip",
+		.data = GUI_V_S("frameskip"),
 		.submenu = 0,
 		.state = 0,
 		.validStates = (const char*[]) {
@@ -53,7 +60,7 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 	};
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Show framerate",
-		.data = "fpsCounter",
+		.data = GUI_V_S("fpsCounter"),
 		.submenu = 0,
 		.state = false,
 		.validStates = (const char*[]) {
@@ -63,7 +70,7 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 	};
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Show status OSD",
-		.data = "showOSD",
+		.data = GUI_V_S("showOSD"),
 		.submenu = 0,
 		.state = true,
 		.validStates = (const char*[]) {
@@ -73,7 +80,7 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 	};
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Autosave state",
-		.data = "autosave",
+		.data = GUI_V_S("autosave"),
 		.submenu = 0,
 		.state = true,
 		.validStates = (const char*[]) {
@@ -83,7 +90,7 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 	};
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Autoload state",
-		.data = "autoload",
+		.data = GUI_V_S("autoload"),
 		.submenu = 0,
 		.state = true,
 		.validStates = (const char*[]) {
@@ -93,7 +100,17 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 	};
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Mute",
-		.data = "mute",
+		.data = GUI_V_S("mute"),
+		.submenu = 0,
+		.state = false,
+		.validStates = (const char*[]) {
+			"Off", "On"
+		},
+		.nStates = 2
+	};
+	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
+		.title = "Fast forward mute",
+		.data = GUI_V_S("fastForwardMute"),
 		.submenu = 0,
 		.state = false,
 		.validStates = (const char*[]) {
@@ -103,7 +120,7 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 	};
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Use BIOS if found",
-		.data = "useBios",
+		.data = GUI_V_S("useBios"),
 		.submenu = 0,
 		.state = true,
 		.validStates = (const char*[]) {
@@ -114,26 +131,38 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 #ifdef M_CORE_GBA
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Select GBA BIOS path",
-		.data = "gba.bios",
+		.data = GUI_V_S("gba.bios"),
 	};
 #endif
 #ifdef M_CORE_GB
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Select GB BIOS path",
-		.data = "gb.bios",
+		.data = GUI_V_S("gb.bios"),
 	};
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Select GBC BIOS path",
-		.data = "gbc.bios",
+		.data = GUI_V_S("gbc.bios"),
 	};
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Select SGB BIOS path",
-		.data = "sgb.bios",
+		.data = GUI_V_S("sgb.bios"),
 	};
+	struct GUIMenuItem* palette = GUIMenuItemListAppend(&menu.items);
+	*palette = (struct GUIMenuItem) {
+		.title = "GB palette",
+		.data = GUI_V_S("gb.pal"),
+	};
+	const struct GBColorPreset* colorPresets;
+	palette->nStates = GBColorPresetList(&colorPresets);
+	const char** paletteStates = calloc(palette->nStates, sizeof(char*));
+	for (i = 0; i < palette->nStates; ++i) {
+		paletteStates[i] = colorPresets[i].name;
+	}
+	palette->validStates = paletteStates;
 #endif
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Interframe blending",
-		.data = "interframeBlending",
+		.data = GUI_V_S("interframeBlending"),
 		.submenu = 0,
 		.state = false,
 		.validStates = (const char*[]) {
@@ -144,7 +173,7 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 #if defined(M_CORE_GBA) && (defined(GEKKO) || defined(__SWITCH__) || defined(PSP2))
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Enable GBP features",
-		.data = "gba.forceGbp",
+		.data = GUI_V_S("gba.forceGbp"),
 		.submenu = 0,
 		.state = false,
 		.validStates = (const char*[]) {
@@ -156,7 +185,7 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 #ifdef M_CORE_GB
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Enable SGB features",
-		.data = "sgb.model",
+		.data = GUI_V_S("sgb.model"),
 		.submenu = 0,
 		.state = true,
 		.validStates = (const char*[]) {
@@ -170,7 +199,7 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 	};
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Enable SGB borders",
-		.data = "sgb.borders",
+		.data = GUI_V_S("sgb.borders"),
 		.submenu = 0,
 		.state = true,
 		.validStates = (const char*[]) {
@@ -180,7 +209,7 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 	};
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Crop SGB borders",
-		.data = "sgb.borderCrop",
+		.data = GUI_V_S("sgb.borderCrop"),
 		.submenu = 0,
 		.state = false,
 		.validStates = (const char*[]) {
@@ -189,7 +218,6 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 		.nStates = 2
 	};
 #endif
-	size_t i;
 	const char* mapNames[GUI_MAX_INPUTS + 1];
 	if (runner->keySources) {
 		for (i = 0; runner->keySources[i].id && i < GUI_MAX_INPUTS; ++i) {
@@ -201,7 +229,7 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 		}
 		*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 			.title = "Remap controls",
-			.data = "*REMAP",
+			.data = GUI_V_U(CONFIG_REMAP),
 			.state = 0,
 			.validStates = i ? mapNames : 0,
 			.nStates = i
@@ -212,11 +240,11 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 	}
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Save",
-		.data = "*SAVE",
+		.data = GUI_V_U(CONFIG_SAVE),
 	};
 	*GUIMenuItemListAppend(&menu.items) = (struct GUIMenuItem) {
 		.title = "Cancel",
-		.data = 0,
+		.data = GUI_V_V,
 	};
 	enum GUIMenuExitReason reason;
 	char gbaBiosPath[256] = "";
@@ -229,59 +257,63 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 	struct GUIMenuItem* item;
 	for (i = 0; i < GUIMenuItemListSize(&menu.items); ++i) {
 		item = GUIMenuItemListGetPointer(&menu.items, i);
-		if (!item->validStates || !item->data) {
+		if (!item->validStates || GUIVariantIsVoid(item->data)) {
 			continue;
 		}
-		if (item->stateMappings) {
-			size_t j;
-			for (j = 0; j < item->nStates; ++j) {
-				const struct GUIVariant* v = &item->stateMappings[j];
-				struct GUIVariant test;
-				switch (v->type) {
-				case GUI_VARIANT_VOID:
-					if (!mCoreConfigGetValue(&runner->config, item->data)) {
-						item->state = j;
+		if (GUIVariantIsString(item->data)) {
+			if (item->stateMappings) {
+				size_t j;
+				for (j = 0; j < item->nStates; ++j) {
+					const struct GUIVariant* v = &item->stateMappings[j];
+					struct GUIVariant test;
+					switch (v->type) {
+					case GUI_VARIANT_VOID:
+						if (!mCoreConfigGetValue(&runner->config, item->data.v.s)) {
+							item->state = j;
+							break;
+						}
+						break;
+					case GUI_VARIANT_UNSIGNED:
+						if (mCoreConfigGetUIntValue(&runner->config, item->data.v.s, &test.v.u) && test.v.u == v->v.u) {
+							item->state = j;
+							break;
+						}
+						break;
+					case GUI_VARIANT_INT:
+						if (mCoreConfigGetIntValue(&runner->config, item->data.v.s, &test.v.i) && test.v.i == v->v.i) {
+							item->state = j;
+							break;
+						}
+						break;
+					case GUI_VARIANT_FLOAT:
+						if (mCoreConfigGetFloatValue(&runner->config, item->data.v.s, &test.v.f) && fabsf(test.v.f - v->v.f) <= 1e-3f) {
+							item->state = j;
+							break;
+						}
+						break;
+					case GUI_VARIANT_STRING:
+						test.v.s = mCoreConfigGetValue(&runner->config, item->data.v.s);
+						if (test.v.s && strcmp(test.v.s, v->v.s) == 0) {
+							item->state = j;
+							break;						
+						}
+						break;
+					case GUI_VARIANT_POINTER:
 						break;
 					}
-					break;
-				case GUI_VARIANT_UNSIGNED:
-					if (mCoreConfigGetUIntValue(&runner->config, item->data, &test.v.u) && test.v.u == v->v.u) {
-						item->state = j;
-						break;
-					}
-					break;
-				case GUI_VARIANT_INT:
-					if (mCoreConfigGetIntValue(&runner->config, item->data, &test.v.i) && test.v.i == v->v.i) {
-						item->state = j;
-						break;
-					}
-					break;
-				case GUI_VARIANT_FLOAT:
-					if (mCoreConfigGetFloatValue(&runner->config, item->data, &test.v.f) && fabsf(test.v.f - v->v.f) <= 1e-3f) {
-						item->state = j;
-						break;
-					}
-					break;
-				case GUI_VARIANT_STRING:
-					test.v.s = mCoreConfigGetValue(&runner->config, item->data);
-					if (test.v.s && strcmp(test.v.s, v->v.s) == 0) {
-						item->state = j;
-						break;						
-					}
-					break;
 				}
+			} else {
+				mCoreConfigGetUIntValue(&runner->config, item->data.v.s, &item->state);
 			}
-		} else {
-			mCoreConfigGetUIntValue(&runner->config, item->data, &item->state);
 		}
 	}
 
 	while (true) {
 		reason = GUIShowMenu(&runner->params, &menu, &item);
-		if (reason != GUI_MENU_EXIT_ACCEPT || !item->data) {
+		if (reason != GUI_MENU_EXIT_ACCEPT || GUIVariantIsVoid(item->data)) {
 			break;
 		}
-		if (!strcmp(item->data, "*SAVE")) {
+		if (GUIVariantCompareUInt(item->data, CONFIG_SAVE)) {
 			if (gbaBiosPath[0]) {
 				mCoreConfigSetValue(&runner->config, "gba.bios", gbaBiosPath);
 			}
@@ -296,30 +328,49 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 			}
 			for (i = 0; i < GUIMenuItemListSize(&menu.items); ++i) {
 				item = GUIMenuItemListGetPointer(&menu.items, i);
-				if (!item->validStates || !item->data || ((const char*) item->data)[0] == '*') {
+				if (!item->validStates || !GUIVariantIsString(item->data)) {
 					continue;
 				}
 				if (item->stateMappings) {
 					const struct GUIVariant* v = &item->stateMappings[item->state];
 					switch (v->type) {
 					case GUI_VARIANT_VOID:
-						mCoreConfigSetValue(&runner->config, item->data, NULL);
+						mCoreConfigSetValue(&runner->config, item->data.v.s, NULL);
 						break;
 					case GUI_VARIANT_UNSIGNED:
-						mCoreConfigSetUIntValue(&runner->config, item->data, v->v.u);
+						mCoreConfigSetUIntValue(&runner->config, item->data.v.s, v->v.u);
 						break;
 					case GUI_VARIANT_INT:
-						mCoreConfigSetUIntValue(&runner->config, item->data, v->v.i);
+						mCoreConfigSetUIntValue(&runner->config, item->data.v.s, v->v.i);
 						break;
 					case GUI_VARIANT_FLOAT:
-						mCoreConfigSetFloatValue(&runner->config, item->data, v->v.f);
+						mCoreConfigSetFloatValue(&runner->config, item->data.v.s, v->v.f);
 						break;
 					case GUI_VARIANT_STRING:
-						mCoreConfigSetValue(&runner->config, item->data, v->v.s);
+						mCoreConfigSetValue(&runner->config, item->data.v.s, v->v.s);
+						break;
+					case GUI_VARIANT_POINTER:
 						break;
 					}
+#ifdef M_CORE_GB
+				} else if (GUIVariantCompareString(item->data, "gb.pal")) {
+					const struct GBColorPreset* preset = &colorPresets[item->state];
+					mCoreConfigSetUIntValue(&runner->config, "gb.pal[0]", preset->colors[0] & 0xFFFFFF);
+					mCoreConfigSetUIntValue(&runner->config, "gb.pal[1]", preset->colors[1] & 0xFFFFFF);
+					mCoreConfigSetUIntValue(&runner->config, "gb.pal[2]", preset->colors[2] & 0xFFFFFF);
+					mCoreConfigSetUIntValue(&runner->config, "gb.pal[3]", preset->colors[3] & 0xFFFFFF);
+					mCoreConfigSetUIntValue(&runner->config, "gb.pal[4]", preset->colors[4] & 0xFFFFFF);
+					mCoreConfigSetUIntValue(&runner->config, "gb.pal[5]", preset->colors[5] & 0xFFFFFF);
+					mCoreConfigSetUIntValue(&runner->config, "gb.pal[6]", preset->colors[6] & 0xFFFFFF);
+					mCoreConfigSetUIntValue(&runner->config, "gb.pal[7]", preset->colors[7] & 0xFFFFFF);
+					mCoreConfigSetUIntValue(&runner->config, "gb.pal[8]", preset->colors[8] & 0xFFFFFF);
+					mCoreConfigSetUIntValue(&runner->config, "gb.pal[9]", preset->colors[9] & 0xFFFFFF);
+					mCoreConfigSetUIntValue(&runner->config, "gb.pal[10]", preset->colors[10] & 0xFFFFFF);
+					mCoreConfigSetUIntValue(&runner->config, "gb.pal[11]", preset->colors[11] & 0xFFFFFF);
+					mCoreConfigSetUIntValue(&runner->config, "gb.pal", item->state);
+#endif
 				} else {
-					mCoreConfigSetUIntValue(&runner->config, item->data, item->state);
+					mCoreConfigSetUIntValue(&runner->config, item->data.v.s, item->state);
 				}
 			}
 			if (runner->keySources) {
@@ -333,11 +384,11 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 			mCoreLoadForeignConfig(runner->core, &runner->config);
 			break;
 		}
-		if (!strcmp(item->data, "*REMAP")) {
+		if (GUIVariantCompareUInt(item->data, CONFIG_REMAP)) {
 			mGUIRemapKeys(&runner->params, &runner->core->inputMap, &runner->keySources[item->state]);
 			continue;
 		}
-		if (!strcmp(item->data, "gba.bios")) {
+		if (GUIVariantCompareString(item->data, "gba.bios")) {
 			// TODO: show box if failed
 			if (!GUISelectFile(&runner->params, gbaBiosPath, sizeof(gbaBiosPath), _biosNamed, GBAIsBIOS, NULL)) {
 				gbaBiosPath[0] = '\0';
@@ -345,21 +396,21 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 			continue;
 		}
 #ifdef M_CORE_GB
-		if (!strcmp(item->data, "gb.bios")) {
+		if (GUIVariantCompareString(item->data, "gb.bios")) {
 			// TODO: show box if failed
 			if (!GUISelectFile(&runner->params, gbBiosPath, sizeof(gbBiosPath), _biosNamed, GBIsBIOS, NULL)) {
 				gbBiosPath[0] = '\0';
 			}
 			continue;
 		}
-		if (!strcmp(item->data, "gbc.bios")) {
+		if (GUIVariantCompareString(item->data, "gbc.bios")) {
 			// TODO: show box if failed
 			if (!GUISelectFile(&runner->params, gbcBiosPath, sizeof(gbcBiosPath), _biosNamed, GBIsBIOS, NULL)) {
 				gbcBiosPath[0] = '\0';
 			}
 			continue;
 		}
-		if (!strcmp(item->data, "sgb.bios")) {
+		if (GUIVariantCompareString(item->data, "sgb.bios")) {
 			// TODO: show box if failed
 			if (!GUISelectFile(&runner->params, sgbBiosPath, sizeof(sgbBiosPath), _biosNamed, GBIsBIOS, NULL)) {
 				sgbBiosPath[0] = '\0';
@@ -380,5 +431,8 @@ void mGUIShowConfig(struct mGUIRunner* runner, struct GUIMenuItem* extra, size_t
 			}
 		}
 	}
+#ifdef M_CORE_GB
+	free(paletteStates);
+#endif
 	GUIMenuItemListDeinit(&menu.items);
 }
