@@ -38,6 +38,21 @@ using QOpenGLFunctions_Baseline = QOpenGLFunctions_3_2_Core;
 #endif
 #endif
 
+#ifdef _WIN32
+#include <windows.h>
+#elif defined(Q_OS_MAC)
+#include <OpenGL/OpenGL.h>
+#endif
+#ifdef USE_GLX
+#define GLX_GLXEXT_PROTOTYPES
+typedef struct _XDisplay Display;
+#include <GL/glx.h>
+#include <GL/glxext.h>
+#endif
+#ifdef USE_EGL
+#include <EGL/egl.h>
+#endif
+
 #ifdef _WIN32
 #define OVERHEAD_NSEC 1000000
 #else
@@ -65,6 +80,10 @@ mGLWidget::mGLWidget(QWidget* parent)
 	connect(&m_refresh, &QTimer::timeout, this, static_cast<void (QWidget::*)()>(&QWidget::update));
 }
 
+mGLWidget::~mGLWidget() {
+	// This is needed for unique_ptr<QOpenGLPaintDevice> to work
+}
+
 void mGLWidget::initializeGL() {
 	m_vao = std::make_unique<QOpenGLVertexArrayObject>();
 	m_vao->create();
@@ -94,6 +113,8 @@ void mGLWidget::initializeGL() {
 
 	m_vaoDone = false;
 	m_tex = 0;
+
+	m_paintDev = std::make_unique<QOpenGLPaintDevice>();
 }
 
 bool mGLWidget::finalizeVAO() {
@@ -145,6 +166,23 @@ void mGLWidget::paintGL() {
 	} else {
 		m_refresh.start(17);
 	}
+
+	if (m_showOSD && m_messagePainter) {
+		qreal r = window()->devicePixelRatio();
+		m_paintDev->setDevicePixelRatio(r);
+		m_paintDev->setSize(size() * r);
+		QPainter painter(m_paintDev.get());
+		m_messagePainter->paint(&painter);
+		painter.end();
+	}
+}
+
+void mGLWidget::setMessagePainter(MessagePainter* messagePainter) {
+	m_messagePainter = messagePainter;
+}
+
+void mGLWidget::setShowOSD(bool showOSD) {
+	m_showOSD = showOSD;
 }
 
 DisplayGL::DisplayGL(const QSurfaceFormat& format, QWidget* parent)
@@ -165,6 +203,7 @@ DisplayGL::DisplayGL(const QSurfaceFormat& format, QWidget* parent)
 		m_gl = new mGLWidget;
 		m_gl->setAttribute(Qt::WA_NativeWindow);
 		m_gl->setFormat(format);
+		m_gl->setMessagePainter(messagePainter());
 		QBoxLayout* layout = new QVBoxLayout;
 		layout->addWidget(m_gl);
 		layout->setContentsMargins(0, 0, 0, 0);
@@ -177,11 +216,6 @@ DisplayGL::DisplayGL(const QSurfaceFormat& format, QWidget* parent)
 	m_drawThread.setObjectName("Painter Thread");
 	m_painter->setThread(&m_drawThread);
 
-	m_proxyThread.setObjectName("OpenGL Proxy Thread");
-	m_proxyContext = std::make_unique<QOpenGLContext>();
-	m_proxyContext->setFormat(format);
-	connect(m_painter.get(), &PainterGL::created, this, &DisplayGL::setupProxyThread);
-
 	connect(&m_drawThread, &QThread::started, m_painter.get(), &PainterGL::create);
 	connect(m_painter.get(), &PainterGL::started, this, [this] {
 		m_hasStarted = true;
@@ -196,11 +230,6 @@ DisplayGL::~DisplayGL() {
 	QMetaObject::invokeMethod(m_painter.get(), "destroy", Qt::BlockingQueuedConnection);
 	m_drawThread.exit();
 	m_drawThread.wait();
-
-	if (m_proxyThread.isRunning()) {
-		m_proxyThread.exit();
-		m_proxyThread.wait();
-	}
 }
 
 bool DisplayGL::supportsShaders() const {
@@ -221,6 +250,9 @@ void DisplayGL::startDrawing(std::shared_ptr<CoreController> controller) {
 	m_painter->setContext(controller);
 	m_painter->setMessagePainter(messagePainter());
 	m_context = controller;
+	if (videoProxy()) {
+		videoProxy()->moveToThread(&m_drawThread);
+	}
 
 	lockAspectRatio(isAspectRatioLocked());
 	lockIntegerScaling(isIntegerScalingLocked());
@@ -235,7 +267,7 @@ void DisplayGL::startDrawing(std::shared_ptr<CoreController> controller) {
 	messagePainter()->resize(size(), devicePixelRatio());
 #endif
 
-	CoreController::Interrupter interrupter(controller);
+	CoreController::Interrupter interrupter(m_context);
 	QMetaObject::invokeMethod(m_painter.get(), "start");
 	if (!m_gl) {
 		if (shouldDisableUpdates()) {
@@ -357,6 +389,9 @@ void DisplayGL::interframeBlending(bool enable) {
 
 void DisplayGL::showOSDMessages(bool enable) {
 	Display::showOSDMessages(enable);
+	if (m_gl) {
+		m_gl->setShowOSD(enable);
+	}
 	QMetaObject::invokeMethod(m_painter.get(), "showOSD", Q_ARG(bool, enable));
 }
 
@@ -370,6 +405,10 @@ void DisplayGL::filter(bool filter) {
 	QMetaObject::invokeMethod(m_painter.get(), "filter", Q_ARG(bool, filter));
 }
 
+void DisplayGL::swapInterval(int interval) {
+	QMetaObject::invokeMethod(m_painter.get(), "swapInterval", Q_ARG(int, interval));
+}
+
 void DisplayGL::framePosted() {
 	m_painter->enqueue(m_context->drawContext());
 	QMetaObject::invokeMethod(m_painter.get(), "draw");
@@ -420,34 +459,11 @@ bool DisplayGL::shouldDisableUpdates() {
 void DisplayGL::setVideoProxy(std::shared_ptr<VideoProxy> proxy) {
 	Display::setVideoProxy(proxy);
 	if (proxy) {
-		proxy->moveToThread(&m_proxyThread);
+		proxy->moveToThread(&m_drawThread);
 	}
 	m_painter->setVideoProxy(proxy);
 }
 
-void DisplayGL::setupProxyThread() {
-	m_proxyContext->moveToThread(&m_proxyThread);
-	m_proxySurface.create();
-	connect(&m_proxyThread, &QThread::started, m_proxyContext.get(), [this]() {
-		m_proxyContext->setShareContext(m_painter->shareContext());
-		m_proxyContext->create();
-		m_proxyContext->makeCurrent(&m_proxySurface);
-#if defined(_WIN32) && defined(USE_EPOXY)
-		epoxy_handle_external_wglMakeCurrent();
-#endif
-	});
-	connect(m_painter.get(), &PainterGL::texSwapped, m_proxyContext.get(), [this]() {
-		if (!m_context->hardwareAccelerated()) {
-			return;
-		}
-		if (videoProxy()) {
-			videoProxy()->processData();
-		}
-		m_painter->updateFramebufferHandle();
-	}, Qt::BlockingQueuedConnection);
-	m_proxyThread.start();
-}
-
 int DisplayGL::framebufferHandle() {
 	return m_painter->glTex();
 }
@@ -509,19 +525,15 @@ void PainterGL::create() {
 	mGLES2Context* gl2Backend;
 #endif
 
-	m_paintDev = std::make_unique<QOpenGLPaintDevice>();
+	if (!m_widget) {
+		m_paintDev = std::make_unique<QOpenGLPaintDevice>();
+	}
 
 #if defined(BUILD_GLES2) || defined(BUILD_GLES3)
 	if (m_supportsShaders) {
-		QOpenGLFunctions_Baseline* fn = m_gl->versionFunctions<QOpenGLFunctions_Baseline>();
 		gl2Backend = static_cast<mGLES2Context*>(malloc(sizeof(mGLES2Context)));
 		mGLES2ContextCreate(gl2Backend);
 		m_backend = &gl2Backend->d;
-		fn->glGenTextures(m_bridgeTexes.size(), m_bridgeTexes.data());
-		for (auto tex : m_bridgeTexes) {
-			m_freeTex.enqueue(tex);
-		}
-		m_bridgeTexIn = m_freeTex.dequeue();
 	}
 #endif
 
@@ -543,7 +555,7 @@ void PainterGL::create() {
 #if defined(BUILD_GLES2) || defined(BUILD_GLES3)
 		mGLES2Context* gl2Backend = reinterpret_cast<mGLES2Context*>(painter->m_backend);
 		if (painter->m_widget && painter->supportsShaders()) {
-			QOpenGLFunctions_Baseline* fn = painter->m_gl->versionFunctions<QOpenGLFunctions_Baseline>();
+			QOpenGLFunctions* fn = painter->m_gl->functions();
 			fn->glFinish();
 			painter->m_widget->setTex(painter->m_finalTex[painter->m_finalTexIdx]);
 			painter->m_finalTexIdx ^= 1;
@@ -589,11 +601,9 @@ void PainterGL::destroy() {
 	}
 	makeCurrent();
 #if defined(BUILD_GLES2) || defined(BUILD_GLES3)
-	QOpenGLFunctions_Baseline* fn = m_gl->versionFunctions<QOpenGLFunctions_Baseline>();
 	if (m_shader.passes) {
 		mGLES2ShaderFree(&m_shader);
 	}
-	fn->glDeleteTextures(m_bridgeTexes.size(), m_bridgeTexes.data());
 #endif
 	m_backend->deinit(m_backend);
 	m_gl->doneCurrent();
@@ -634,8 +644,10 @@ void PainterGL::setMessagePainter(MessagePainter* messagePainter) {
 void PainterGL::resize(const QSize& size) {
 	qreal r = m_window->devicePixelRatio();
 	m_size = size;
-	m_paintDev->setSize(m_size * r);
-	m_paintDev->setDevicePixelRatio(r);
+	if (m_paintDev) {
+		m_paintDev->setSize(m_size * r);
+		m_paintDev->setDevicePixelRatio(r);
+	}
 	if (m_started && !m_active) {
 		forceDraw();
 	}
@@ -670,6 +682,32 @@ void PainterGL::filter(bool filter) {
 	}
 }
 
+void PainterGL::swapInterval(int interval) {
+	if (!m_started) {
+		return;
+	}
+	m_swapInterval = interval;
+#ifdef Q_OS_WIN
+	wglSwapIntervalEXT(interval);
+#elif defined(Q_OS_MAC)
+	CGLSetParameter(CGLGetCurrentContext(), kCGLCPSwapInterval, &interval);
+#else
+#ifdef USE_GLX
+	if (QGuiApplication::platformName() == "xcb") {
+		::Display* display = glXGetCurrentDisplay();
+		GLXDrawable drawable = glXGetCurrentDrawable();
+		glXSwapIntervalEXT(display, drawable, interval);
+	}
+#endif
+#ifdef USE_EGL
+	if (QGuiApplication::platformName().contains("egl")) {
+		EGLDisplay display = eglGetCurrentDisplay();
+		eglSwapInterval(display, interval);
+	}
+#endif
+#endif
+}
+
 #ifndef GL_DEBUG_OUTPUT_SYNCHRONOUS
 #define GL_DEBUG_OUTPUT_SYNCHRONOUS 0x8242
 #endif
@@ -680,7 +718,7 @@ void PainterGL::start() {
 	if (glContextHasBug(OpenGLBug::GLTHREAD_BLOCKS_SWAP)) {
 		// Suggested on Discord as a way to strongly hint that glthread should be disabled
 		// See https://gitlab.freedesktop.org/mesa/mesa/-/issues/8035
-		QOpenGLFunctions_Baseline* fn = m_gl->versionFunctions<QOpenGLFunctions_Baseline>();
+		QOpenGLFunctions* fn = m_gl->functions();
 		fn->glEnable(GL_DEBUG_OUTPUT_SYNCHRONOUS);
 	}
 #endif
@@ -691,16 +729,16 @@ void PainterGL::start() {
 	}
 #endif
 	resizeContext();
-	m_context->addFrameAction(std::bind(&PainterGL::swapTex, this));
 
 	m_buffer = nullptr;
 	m_active = true;
 	m_started = true;
+	swapInterval(1);
 	emit started();
 }
 
 void PainterGL::draw() {
-	if (!m_started || (m_queue.isEmpty() && m_queueTex.isEmpty())) {
+	if (!m_started || m_queue.isEmpty()) {
 		return;
 	}
 
@@ -726,12 +764,16 @@ void PainterGL::draw() {
 		}
 		return;
 	}
+	int wantSwap = sync->audioWait || sync->videoFrameWait;
+	if (m_swapInterval != wantSwap) {
+		swapInterval(wantSwap);
+	}
 	dequeue();
 	bool forceRedraw = true;
 	if (!m_delayTimer.isValid()) {
 		m_delayTimer.start();
 	} else {
-		if (sync->audioWait || sync->videoFrameWait) {
+		if (wantSwap) {
 			while (m_delayTimer.nsecsElapsed() + OVERHEAD_NSEC < 1000000000 / sync->fpsTarget) {
 				QThread::usleep(500);
 			}
@@ -783,6 +825,11 @@ void PainterGL::doStop() {
 	}
 	m_backend->clear(m_backend);
 	m_backend->swap(m_backend);
+	if (m_videoProxy) {
+		m_videoProxy->reset();
+		m_videoProxy->moveToThread(m_window->thread());
+		m_videoProxy.reset();
+	}
 }
 
 void PainterGL::pause() {
@@ -802,7 +849,7 @@ void PainterGL::performDraw() {
 		m_backend->postFrame(m_backend, m_buffer);
 	}
 	m_backend->drawFrame(m_backend);
-	if (m_showOSD && m_messagePainter) {
+	if (m_showOSD && m_messagePainter && m_paintDev && !glContextHasBug(OpenGLBug::IG4ICD_CRASH)) {
 		m_painter.begin(m_paintDev.get());
 		m_messagePainter->paint(&m_painter);
 		m_painter.end();
@@ -810,33 +857,22 @@ void PainterGL::performDraw() {
 }
 
 void PainterGL::enqueue(const uint32_t* backing) {
-	if (!backing) {
-		return;
-	}
 	QMutexLocker locker(&m_mutex);
 	uint32_t* buffer = nullptr;
-	if (m_free.isEmpty()) {
-		buffer = m_queue.dequeue();
-	} else {
-		buffer = m_free.takeLast();
-	}
-	if (buffer) {
-		QSize size = m_context->screenDimensions();
-		memcpy(buffer, backing, size.width() * size.height() * BYTES_PER_PIXEL);
+	if (backing) {
+		if (m_free.isEmpty()) {
+			buffer = m_queue.dequeue();
+		} else {
+			buffer = m_free.takeLast();
+		}
+		if (buffer) {
+			QSize size = m_context->screenDimensions();
+			memcpy(buffer, backing, size.width() * size.height() * BYTES_PER_PIXEL);
+		}
 	}
 	m_queue.enqueue(buffer);
 }
 
-void PainterGL::enqueue(GLuint tex) {
-	QMutexLocker locker(&m_mutex);
-	if (m_freeTex.isEmpty()) {
-		m_bridgeTexIn = m_queueTex.dequeue();
-	} else {
-		m_bridgeTexIn = m_freeTex.takeLast();
-	}
-	m_queueTex.enqueue(tex);
-}
-
 void PainterGL::dequeue() {
 	QMutexLocker locker(&m_mutex);
 	if (!m_queue.isEmpty()) {
@@ -846,19 +882,6 @@ void PainterGL::dequeue() {
 		}
 		m_buffer = buffer;
 	}
-
-	if (!m_queueTex.isEmpty()) {
-		if (m_bridgeTexOut != std::numeric_limits<GLuint>::max()) {
-			m_freeTex.enqueue(m_bridgeTexOut);
-		}
-		m_bridgeTexOut = m_queueTex.dequeue();
-#if defined(BUILD_GLES2) || defined(BUILD_GLES3)
-		if (supportsShaders()) {
-			mGLES2Context* gl2Backend = reinterpret_cast<mGLES2Context*>(m_backend);
-			gl2Backend->tex = m_bridgeTexOut;
-		}
-#endif
-	}
 }
 
 void PainterGL::dequeueAll(bool keep) {
@@ -879,19 +902,6 @@ void PainterGL::dequeueAll(bool keep) {
 		m_free.append(m_buffer);
 		m_buffer = nullptr;
 	}
-
-	m_queueTex.clear();
-	m_freeTex.clear();
-	for (auto tex : m_bridgeTexes) {
-		if (keep && tex == m_bridgeTexIn) {
-			continue;
-		}
-		m_freeTex.enqueue(tex);
-	}
-	if (!keep) {
-		m_bridgeTexIn = m_freeTex.dequeue();
-		m_bridgeTexOut = std::numeric_limits<GLuint>::max();
-	}
 }
 
 void PainterGL::setVideoProxy(std::shared_ptr<VideoProxy> proxy) {
@@ -915,8 +925,9 @@ void PainterGL::setShaders(struct VDir* dir) {
 		mGLES2ShaderDetach(reinterpret_cast<mGLES2Context*>(m_backend));
 		mGLES2ShaderFree(&m_shader);
 	}
-	mGLES2ShaderLoad(&m_shader, dir);
-	mGLES2ShaderAttach(reinterpret_cast<mGLES2Context*>(m_backend), static_cast<mGLES2Shader*>(m_shader.passes), m_shader.nPasses);
+	if (mGLES2ShaderLoad(&m_shader, dir)) {
+		mGLES2ShaderAttach(reinterpret_cast<mGLES2Context*>(m_backend), static_cast<mGLES2Shader*>(m_shader.passes), m_shader.nPasses);
+	}
 
 	if (!m_started) {
 		m_gl->doneCurrent();
@@ -971,31 +982,4 @@ QOpenGLContext* PainterGL::shareContext() {
 	}
 }
 
-void PainterGL::updateFramebufferHandle() {
-	QOpenGLFunctions_Baseline* fn = m_gl->versionFunctions<QOpenGLFunctions_Baseline>();
-	// TODO: Figure out why glFlush doesn't work here on Intel/Windows
-	if (glContextHasBug(OpenGLBug::CROSS_THREAD_FLUSH)) {
-		fn->glFinish();
-	} else {
-		fn->glFlush();
-	}
-
-	CoreController::Interrupter interrupter(m_context);
-	if (!m_context->hardwareAccelerated()) {
-		return;
-	}
-	enqueue(m_bridgeTexIn);
-	m_context->setFramebufferHandle(m_bridgeTexIn);
-}
-
-void PainterGL::swapTex() {
-	if (!m_started) {
-		return;
-	}
-
-	CoreController::Interrupter interrupter(m_context);
-	emit texSwapped();
-	m_context->addFrameAction(std::bind(&PainterGL::swapTex, this));
-}
-
 #endif
