@@ -5,6 +5,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 #include "utils.h"
 
+#include <QCoreApplication>
+#include <QKeySequence>
 #include <QObject>
 
 namespace QGBA {
@@ -59,4 +61,27 @@ bool convertAddress(const QHostAddress* input, Address* output) {
 	return true;
 }
 
+QString keyName(int key) {
+	switch (key) {
+#ifndef Q_OS_MAC
+	case Qt::Key_Shift:
+		return QCoreApplication::translate("QShortcut", "Shift");
+	case Qt::Key_Control:
+		return QCoreApplication::translate("QShortcut", "Control");
+	case Qt::Key_Alt:
+		return QCoreApplication::translate("QShortcut", "Alt");
+	case Qt::Key_Meta:
+		return QCoreApplication::translate("QShortcut", "Meta");
+#endif
+	case Qt::Key_Super_L:
+		return QObject::tr("Super (L)");
+	case Qt::Key_Super_R:
+		return QObject::tr("Super (R)");
+	case Qt::Key_Menu:
+		return QObject::tr("Menu");
+	default:
+		return QKeySequence(key).toString(QKeySequence::NativeText);
+	}
+}
+
 }
