@@ -115,7 +115,7 @@ static float gyroZ = 0;
 static float tiltX = 0;
 static float tiltY = 0;
 
-static struct GBAStereoSample audioBuffer[N_BUFFERS][BUFFER_SIZE / 4] __attribute__((__aligned__(0x1000)));
+static struct mStereoSample audioBuffer[N_BUFFERS][BUFFER_SIZE / 4] __attribute__((__aligned__(0x1000)));
 
 static enum ScreenMode {
 	SM_PA,
@@ -584,7 +584,7 @@ static void _postAudioBuffer(struct mAVStream* stream, blip_t* left, blip_t* rig
 		blip_clear(right);
 		return;
 	}
-	struct GBAStereoSample* samples = audioBuffer[audioBufferActive];
+	struct mStereoSample* samples = audioBuffer[audioBufferActive];
 	blip_read_samples(left, &samples[0].left, SAMPLES, true);
 	blip_read_samples(right, &samples[0].right, SAMPLES, true);
 	audoutAppendAudioOutBuffer(&audoutBuffer[audioBufferActive]);
@@ -674,6 +674,24 @@ static void _guiFinish(void) {
 	GUIFontDrawSubmit(font);
 }
 
+static enum GUIKeyboardStatus _keyboardRun(struct GUIKeyboardParams* keyboard) {
+	SwkbdConfig swkbd;
+	swkbdCreate(&swkbd, 0);
+	swkbdConfigMakePresetDefault(&swkbd);
+	swkbdConfigSetStringLenMax(&swkbd, keyboard->maxLen);
+	swkbdConfigSetInitialText(&swkbd, keyboard->result);
+	swkbdConfigSetHeaderText(&swkbd, keyboard->title);
+	swkbdConfigSetReturnButtonFlag(&swkbd, keyboard->multiline);
+
+	Result rc = swkbdShow(&swkbd, keyboard->result, sizeof(keyboard->result));
+	swkbdClose(&swkbd);
+	if (R_SUCCEEDED(rc)) {
+		return GUI_KEYBOARD_DONE;
+	} else {
+		return GUI_KEYBOARD_CANCEL;
+	}
+}
+
 static void glInit(void) {
 	glViewport(0, 1080 - vheight, vwidth, vheight);
 	glClearColor(0.f, 0.f, 0.f, 1.f);
@@ -865,6 +883,7 @@ int main(int argc, char* argv[]) {
 			_pollInput, _pollCursor,
 			_batteryState,
 			_guiPrepare, _guiFinish,
+			_keyboardRun,
 		},
 		.keySources = (struct GUIInputKeys[]) {
 			{
@@ -905,7 +924,7 @@ int main(int argc, char* argv[]) {
 		.configExtra = (struct GUIMenuItem[]) {
 			{
 				.title = "Screen mode",
-				.data = "screenMode",
+				.data = GUI_V_S("screenMode"),
 				.submenu = 0,
 				.state = SM_PA,
 				.validStates = (const char*[]) {
@@ -917,7 +936,7 @@ int main(int argc, char* argv[]) {
 			},
 			{
 				.title = "Fast forward cap",
-				.data = "fastForwardCap",
+				.data = GUI_V_S("fastForwardCap"),
 				.submenu = 0,
 				.state = 7,
 				.validStates = (const char*[]) {
@@ -947,7 +966,7 @@ int main(int argc, char* argv[]) {
 			},
 			{
 				.title = "GPU-accelerated renderer",
-				.data = "hwaccelVideo",
+				.data = GUI_V_S("hwaccelVideo"),
 				.submenu = 0,
 				.state = 0,
 				.validStates = (const char*[]) {
@@ -958,7 +977,7 @@ int main(int argc, char* argv[]) {
 			},
 			{
 				.title = "Hi-res scaling (requires GPU rendering)",
-				.data = "videoScale",
+				.data = GUI_V_S("videoScale"),
 				.submenu = 0,
 				.state = 0,
 				.validStates = (const char*[]) {
@@ -981,7 +1000,7 @@ int main(int argc, char* argv[]) {
 			},
 			{
 				.title = "Use built-in brightness sensor for Boktai",
-				.data = "useLightSensor",
+				.data = GUI_V_S("useLightSensor"),
 				.submenu = 0,
 				.state = illuminanceAvailable,
 				.validStates = (const char*[]) {
