@@ -28,6 +28,8 @@
 #include <getopt.h>
 #endif
 
+#include <setjmp.h>
+#include <signal.h>
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -37,6 +39,7 @@
 #define LOG_THRESHOLD 1000000
 
 static const struct option longOpts[] = {
+	{ "4up",        no_argument, 0, '4' },
 	{ "base",       required_argument, 0, 'b' },
 	{ "diffs",      no_argument, 0, 'd' },
 	{ "help",       no_argument, 0, 'h' },
@@ -52,7 +55,7 @@ static const struct option longOpts[] = {
 	{ 0, 0, 0, 0 }
 };
 
-static const char shortOpts[] = "b:dhj:no:qRrvx";
+static const char shortOpts[] = "4b:dhj:no:qRrvx";
 
 enum CInemaStatus {
 	CI_PASS,
@@ -79,6 +82,7 @@ struct CInemaTest {
 	unsigned totalFrames;
 	uint64_t totalDistance;
 	uint64_t totalPixels;
+	jmp_buf errorCtx;
 };
 
 struct CInemaImage {
@@ -98,7 +102,6 @@ struct StringBuilder {
 	struct StringList lines;
 	struct StringList partial;
 	unsigned repeat;
-
 };
 
 struct CInemaLogStream {
@@ -112,6 +115,7 @@ static char base[PATH_MAX] = {0};
 static char outdir[PATH_MAX] = {'.'};
 static bool dryRun = false;
 static bool diffs = false;
+static bool is4Up = false;
 static enum CInemaRebaseline rebaseline = CI_R_NONE;
 static enum CInemaRebaseline xbaseline = CI_R_NONE;
 static int verbosity = 0;
@@ -242,6 +246,9 @@ static bool parseCInemaArgs(int argc, char* const* argv) {
 				return false;
 			}
 			break;
+		case '4':
+			is4Up = true;
+			break;
 		case 'b':
 			strlcpy(base, optarg, sizeof(base));
 			// TODO: Verify path exists
@@ -862,7 +869,42 @@ static bool _compareImages(struct CInemaTest* restrict test, const struct CInema
 	return !failed;
 }
 
-void _writeDiffSet(struct CInemaImage* expected, const char* name, uint8_t* diff, int frame, int max, bool xfail) {
+static void _write4UpDiff(const struct CInemaImage* expected, const struct CInemaImage* result, const char* name, uint8_t* diff, int frame, int max) {
+	struct CInemaImage out = {
+		.width = expected->width * 2,
+		.height = expected->height * 2,
+		.stride = expected->width * 2,
+	};
+	out.data = malloc(out.height * out.stride * 4);
+	uint32_t* outdata = out.data;
+	size_t x;
+	size_t y;
+	for (y = 0; y < expected->height; ++y) {
+		size_t base = y * out.stride;
+		size_t inbase = y * expected->stride;
+		memcpy(&outdata[base], &((uint32_t*) expected->data)[inbase], expected->width * 4);
+		memcpy(&outdata[base + expected->width], &((uint32_t*) result->data)[y * result->stride], expected->width * 4);
+		memcpy(&outdata[base + expected->height * out.stride], &diff[inbase * 4], expected->width * 4);
+		for (x = 0; x < expected->width; ++x) {
+			size_t pix = (expected->stride * y + x) * 4;
+			size_t outpix = base + expected->height * out.stride + expected->width + x;
+			uint8_t* outdiff = (uint8_t*) &outdata[outpix];
+#ifndef __BIG_ENDIAN__
+			outdiff[0] = diff[pix + 0] * 255 / max;
+			outdiff[1] = diff[pix + 1] * 255 / max;
+			outdiff[2] = diff[pix + 2] * 255 / max;
+#else
+			outdiff[1] = diff[pix + 1] * 255 / max;
+			outdiff[2] = diff[pix + 2] * 255 / max;
+			outdiff[3] = diff[pix + 3] * 255 / max;
+#endif
+		}
+	}
+	_writeDiff(name, &out, frame, "4up");
+	free(out.data);
+}
+
+static void _writeDiffSet(const struct CInemaImage* expected, const char* name, uint8_t* diff, int frame, int max, bool xfail) {
 	struct CInemaImage outdiff = {
 		.data = diff,
 		.width = expected->width,
@@ -1084,6 +1126,9 @@ void CInemaTestRun(struct CInemaTest* test) {
 	bool xdiff = false;
 	for (frame = 0; limit; ++frame, --limit) {
 		_updateInput(core, frame, &input);
+		if (setjmp(test->errorCtx)) {
+			break;
+		}
 		core->runFrame(core);
 		++test->totalFrames;
 		unsigned frameCounter = core->frameCounter(core);
@@ -1155,8 +1200,12 @@ void CInemaTestRun(struct CInemaTest* test) {
 			}
 			if (diff) {
 				if (failed) {
-					_writeDiff(test->name, &image, frame, "result");
-					_writeDiffSet(&expected, test->name, diff, frame, max, false);
+					if (is4Up) {
+						_write4UpDiff(&expected, &image, test->name, diff, frame, max);
+					} else {
+						_writeDiff(test->name, &image, frame, "result");
+						_writeDiffSet(&expected, test->name, diff, frame, max, false);
+					}
 				}
 				free(diff);
 				diff = NULL;
@@ -1349,6 +1398,21 @@ void _log(struct mLogger* log, int category, enum mLogLevel level, const char* f
 	CIerr(0, "[%s] %s\n", mLogCategoryName(category), buffer);
 }
 
+#if defined(_POSIX_C_SOURCE) && _POSIX_C_SOURCE >= 200112L
+static void _signalHandler(int signal, siginfo_t* info, void* context) {
+	UNUSED(info);
+	UNUSED(context);
+	struct CInemaTest* test = ThreadLocalGetValue(currentTest);
+	if (test) {
+		test->status = CI_ERROR;
+		CIerr(0, "Test %s crashed with signal %i\n", test->name, signal);
+	} else {
+		CIerr(0, "Thread crashed with signal %i\n", signal);
+	}
+	longjmp(test->errorCtx, -1);
+}
+#endif
+
 int main(int argc, char** argv) {
 	ThreadLocalInitKey(&logStream);
 	ThreadLocalSetKey(logStream, NULL);
@@ -1420,7 +1484,7 @@ int main(int argc, char** argv) {
 	}
 
 	if (CInemaTestListSize(&tests) == 0) {
-		CIlog(1, "No tests found.");
+		CIlog(1, "No tests found.\n");
 		status = 1;
 	} else {
 		reduceTestList(&tests);
@@ -1442,12 +1506,34 @@ int main(int argc, char** argv) {
 	} else {
 		MutexInit(&jobMutex);
 		int i;
+
+#if defined(_POSIX_C_SOURCE) && _POSIX_C_SOURCE >= 200112L
+		struct sigaction sa = {
+			.sa_flags = SA_SIGINFO,
+			.sa_sigaction = _signalHandler,
+		};
+		sigemptyset(&sa.sa_mask);
+
+		sigaction(SIGSEGV, &sa, NULL);
+		sigaction(SIGBUS, &sa, NULL);
+		sigaction(SIGFPE, &sa, NULL);
+		sigaction(SIGILL, &sa, NULL);
+#endif
+
 		for (i = 0; i < jobs; ++i) {
 			ThreadCreate(&jobThreads[i], CInemaJob, &tests);
 		}
 		for (i = 0; i < jobs; ++i) {
 			ThreadJoin(&jobThreads[i]);
 		}
+
+#if defined(_POSIX_C_SOURCE) && _POSIX_C_SOURCE >= 200112L
+		signal(SIGSEGV, SIG_DFL);
+		signal(SIGBUS, SIG_DFL);
+		signal(SIGFPE, SIG_DFL);
+		signal(SIGILL, SIG_DFL);
+#endif
+
 		MutexDeinit(&jobMutex);
 		status = jobStatus;
 	}
