@@ -98,7 +98,7 @@ static const char* _socketLuaSource =
 	"        local cbid = self._nextCallback\n"
 	"        self._nextCallback = cbid + 1\n"
 	"        self._callbacks[event][cbid] = callback\n"
-	"        return id\n"
+	"        return cbid\n"
 	"      end,\n"
 	"      remove = function(self, cbid)\n"
 	"        for _, group in pairs(self._callbacks) do\n"
@@ -131,7 +131,7 @@ static const char* _socketLuaSource =
 	"      end,\n"
 	"      connect = function(self, address, port)\n"
 	"        local status = self._s:connect(address, port)\n"
-	"        return socket._wrap(status)\n"
+	"        return self:_hook(status)\n"
 	"      end,\n"
 	"      listen = function(self, backlog)\n"
 	"        local status = self._s:listen(backlog or 1)\n"
@@ -584,13 +584,18 @@ struct mScriptValue* _luaCoerceTable(struct mScriptEngineContextLua* luaContext)
 			return false;
 		}
 		mScriptTableInsert(table, key, value);
-		mScriptValueDeref(key);
+		if (key->type != mSCRIPT_TYPE_MS_STR) {
+			// Strings are added to the ref pool, so we need to keep it
+			// ref'd to prevent it from being collected prematurely
+			mScriptValueDeref(key);
+		}
 		mScriptValueDeref(value);
 	}
 	lua_pop(luaContext->lua, 1);
 
 	size_t len = mScriptTableSize(table);
 	if (!isList || !len) {
+		mScriptContextFillPool(luaContext->d.context, table);
 		return table;
 	}
 
@@ -600,6 +605,7 @@ struct mScriptValue* _luaCoerceTable(struct mScriptEngineContextLua* luaContext)
 		struct mScriptValue* value = mScriptTableLookup(table, &mSCRIPT_MAKE_S64(i));
 		if (!value) {
 			mScriptValueDeref(list);
+			mScriptContextFillPool(luaContext->d.context, table);
 			return table;
 		}
 		mScriptValueWrap(value, mScriptListAppend(list->value.list));
